#include "stdafx.h"
#include <assert.h>
#include "channel_hook_impl.h"
#include "channel_hook_general.h"
#include "tls_manager.h"

STDMETHODIMP CChannelHook::QueryInterface(
    REFIID riid,    //interface IID
    void **ppv	    //returned value - interface pointer
    )
{
    HRESULT res_status = S_OK;
    if (riid == IID_IUnknown)
    {
        *ppv = static_cast<IUnknown*>(this);
    } else 
    {
	if (riid == IID_IChannelHook)
	{
	    *ppv = static_cast<IChannelHook*>(this);
	} else
	{
	    *ppv = NULL;
	};
    };
    if (*ppv != NULL)
    {
	reinterpret_cast<IUnknown*>(*ppv)->AddRef();
    } else
    {
	res_status = E_NOINTERFACE;
    };
    return res_status;
}

STDMETHODIMP_(ULONG) CChannelHook::AddRef(void)
{
    return 2;
}

STDMETHODIMP_(ULONG) CChannelHook::Release(void)
{
    return 1;
}

STDMETHODIMP_(void) CChannelHook::ClientGetSize(
    REFGUID uExtent,	//IID of the channel hook
    REFIID riid,	//????
    ULONG *pDataSize	//returned value - size of data buffer
    )
{
    assert(uExtent == CHANNEL_HOOK_ID);
    *pDataSize = sizeof(SNodeInfo);
}

STDMETHODIMP_(void) CChannelHook::ClientFillBuffer(
    REFGUID uExtent,	//IID of the channel hook
    REFIID riid,	//???    
    ULONG *pDataSize,	//size of the data buffer
    void *pDataBuffer	//pointer to data buffer
    )
{
    CThreadContext* thread_context = tls_manager.get_current_thread_context();

    CCallContext* client_info = new CCallContext();
    client_info->fill_user_token_info();
    memcpy(pDataBuffer, client_info->get_node_info(), sizeof(SNodeInfo));

    thread_context->sent_client_info.push(client_info);
}

STDMETHODIMP_(void) CChannelHook::ClientNotify(
    REFGUID uExtent,	//IID of the channel hook
    REFIID riid,	//????
    ULONG cbDataSize,	//size of the received data buffer
    void *pDataBuffer,	//pointer to the data buffer
    DWORD lDataRep,	//????
    HRESULT hrFault	//?????
    )
{
    CThreadContext* thread_context = tls_manager.get_current_thread_context();
    //delete previously sent client information
    delete thread_context->sent_client_info.top();
    thread_context->sent_client_info.pop();
}

STDMETHODIMP_(void) CChannelHook::ServerNotify(
    REFGUID uExtent,	//IID of the channel hook
    REFIID riid,	//?????
    ULONG cbDataSize,	//size of the received data buffer
    void *pDataBuffer,	//pointer to the data buffer
    DWORD lDataRep	//??????
    )
{
    CThreadContext* thread_context = tls_manager.get_current_thread_context();
    if(cbDataSize == sizeof(SNodeInfo))
    {
	SNodeInfo* node_info = reinterpret_cast<SNodeInfo*>(pDataBuffer);

	CCallContext* call_info = new CCallContext();
	if (node_info != NULL)
	{
	    *(call_info->get_node_info()) = *node_info;
	};
	thread_context->recieved_client_info.push(call_info);
    } else
    {
	thread_context->recieved_client_info.push(NULL);
    };
}

STDMETHODIMP_(void) CChannelHook::ServerGetSize(
    REFGUID uExtent,	//IID of the channel hook
    REFIID riid,	//????
    HRESULT hrFault,	//????
    ULONG *pDataSize	// return value - size of the data buffer
    )
{
    assert(uExtent == CHANNEL_HOOK_ID);
    *pDataSize = 0;

    CThreadContext* thread_context = tls_manager.get_current_thread_context();
//	thread_context->recieved_client_info->clear();
    delete thread_context->recieved_client_info.top();
    thread_context->recieved_client_info.pop();
}

STDMETHODIMP_(void) CChannelHook::ServerFillBuffer(REFGUID uExtent, REFIID riid, ULONG *pDataSize, void *pDataBuffer, HRESULT hrFault)
{
}
